(*******************************************************************)
(*  This is part of SymbolicKAT, it is distributed under the       *)
(*  terms of the GNU Lesser General Public License version 3       *)
(*           (see file LICENSE for more details)                   *)
(*                                                                 *)
(*  Copyright 2014: Damien Pous. (CNRS, LIP - ENS Lyon, UMR 5668)  *)
(*******************************************************************)

open Common
open Kat
open Bdd

(* slight optimisation: do not multiply by one on the left-hand side *)
let dot' y x = match x with Tst t when t=Bdd.top -> y | _ -> Dot(x,y)

let deriv mem = 
  let zer = constant mem Set.empty in
  let one = constant mem (Set.single one) in
  let rec deriv = function
  | Pls(x,y) -> Span.merge (binary mem Set.union) (deriv x) (deriv y)
  | Dot(x,y) -> Span.merge (binary mem Set.union)
    (Span.map (unary mem (Set.map (dot' y))) (deriv x))
    (Span.map (times mem zer (epsilon x)) (deriv y))
  | Str x as sx -> Span.map (unary mem (Set.map (dot' sx))) (deriv x)
  | Var i -> Span.single i one zer
  | Tst _ -> Span.empty zer
  in deriv

let nfa () = 
  let m = Bdd.init() in 
  Automata.SNFA.({ m; t = deriv m; o = epsilon })
