(*******************************************************************)
(*  This is part of SymbolicKAT, it is distributed under the       *)
(*  terms of the GNU Lesser General Public License version 3       *)
(*           (see file LICENSE for more details)                   *)
(*                                                                 *)
(*  Copyright 2014: Damien Pous. (CNRS, LIP - ENS Lyon, UMR 5668)  *)
(*******************************************************************)

(** Types for the various kind of manipulated automata *)

open Common

(** symbolic deterministic automata *)
module SDFA = struct
  type ('s,'o) t = {
    m: 's Bdd.mem;
    t: 's -> 's Bdd.node span;
    o: 's -> 'o
  }
end
type ('s,'o) sdfa = ('s,'o) SDFA.t

(** symbolic non-deterministic automata *)
module SNFA = struct
  type ('s,'o) t = {
    m: 's set Bdd.mem;
    t: 's -> 's set Bdd.node span;
    o: 's -> 'o
  }
end
type ('s,'o) snfa = ('s,'o) SNFA.t

(** non-deterministic automata with epsilon transitions *)
module SENFA = struct
  type ('s,'o) t = {
    e: ('s*Bdd.formula*'s) list;
    t: ('s*var*'s) list;
    o: 's list
  }
end
type ('s,'o) senfa = ('s,'o) SENFA.t
