(*******************************************************************)
(*  This is part of SymbolicKAT, it is distributed under the       *)
(*  terms of the GNU Lesser General Public License version 3       *)
(*           (see file LICENSE for more details)                   *)
(*                                                                 *)
(*  Copyright 2014: Damien Pous. (CNRS, LIP - ENS Lyon, UMR 5668)  *)
(*******************************************************************)

(** a few sanity checks *)

open Symbolic_kat

let _ = check `E "r(sr)*" "(rs)*r"
let _ = check `E "(rs)*" "1+r(sr)*s"
let _ = check `E "(r+s)*" "r*(sr*)*"
let _ = check `L "(rs)*" "r*(sr*)*"
let _ = check `D "(rs)*" "r*sr*"

let _ = check `E "1" "a*"
let _ = check `R "a" "ab"
let _ = check `E "a+bc" "cb+a"
let _ = check `E "aar+bc" "cb+ar+bc"
let _ = check `L "(ara)*" "1+ar*a"
let _ = check `D "(ra)*" "a+r*a"
let _ = check `D "(ara)*" "1+ar*ba"
let _ = check `E "1" "1+a"
let _ = check `L "1" "1+ar"
let _ = check `L "1" "1+ra"
let _ = check `R "1" "0"
let _ = check `E "1" "a+!a"
let _ = check `E "0" "a!a"

let _ = check `E "p*" "(1+p)*"
let _ = check `E "p*" "(a+p+b)*"
let _ = check `E "p*" "(1+p*)*"
let _ = check `E "(p+q)*" "((1+p)(1+q))*"
let _ = check `E "(p+q)*" "((1+p)q*)*"
let _ = check `R "(p+q)*" "((a+p)(b+q))*"

let _ = check `E "!b+!bb" "!b"
let _ = check `E "!b+!bbr" "!b"
let _ = check `E "!aar" "0"
let _ = check `E "!b+!bbr(br)*" "!b"
let _ = check `E "!b(1+br(br)*)" "!b"
let _ = check `E "!b(br)*" "!b"
let _ = check `E "(bc+!b!c)(brs+!brt)" "bcrs+!b!crt"
let _ = check `D "(bc+!b!c)(brs+!brt)" "(bc+!b!c)r(cs+!ct)" (* need rc=cr *)

let _ = check `E "(ap!a)*" "1+ap!a"
let _ = check `E "(a(ap)*!a)*!a" "(ap)*!a"
let _ = check `E "(ap)*!a(ap)*!a" "(ap)*!a"

let _ = check `E
  "(p+q+r+s+t)* + abp*(q+!ars*+b(ct+s)*)"
  "(p+q+r+s+t)* + abcq*(q+!asr*+b(cs+t)*)"

let _ = check `E
  "(p+q+r+s+t)* + abp*(q+!ars*+b(ct+s)*(t*r!cb+sss))"
  "(p+q+r+s+t)* + abcq*(q+!asr*+b(cs+t)*(t*r!cd+sts)+!bs*t*)"

let _ = check `E
  "(p+q+r+s+t)* + abp*(q+!ar*(brr+!dt*s*(ar*(cs+tt)*))s*+b(ct+s)*(t*r!cb+sss))"
  "(p+q+r+s+t)* + abcq*(q+!asr*+b(cs+t)*!dt*s*(ar*(sc+ttt)*)(t*r!cd+sts)+!bs*t*)"

let _ = Stats.print Format.std_formatter
let _ = Stats.reset()
